import sys
import warnings
import re
import os
from datetime import datetime
# Autor: Thomas Weber
# Open file with proper filename
try:
    try:
        filename = sys.argv[1]
    except IndexError:
        warnings.warn('No filename supplied. Using movielist.txt')
        filename = 'movielist.txt'
    file_in = iter(open(filename, 'r'))

    try:
        orig_filename_out = sys.argv[2]
        # if re.match(r'.html$', orig_filename_out):
        if orig_filename_out[-5:-1] is '.html':
            orig_filename_out += '.html'
        c = 0
        filename_out = orig_filename_out
        while os.path.isfile(filename_out):
            filename_out = '%s-%s.html' % (orig_filename_out[0:-5], c)
            c += 1
        file_out = open(filename_out, 'w')
    except IndexError:
        file_out = None
except IOError:
    warnings.warn('Couldn\'t open file. Exiting...')
    sys.exit()


class Movie(object):
    """ Movie class, 'cause reasons """

    def __init__(self, args):
        try:
            self.name = args['name']
        except KeyError:
            raise TypeError('Movie without name supplied')

        self.date = datetime.strptime(args.get('date', None), '%d.%m.%Y %H:%M')
        self.file = args.get('file', None)

    def __str__(self):
        return "%s (%s) - %s" % (self.name, self.date, self.file,)

# Read movies from file
data = {'movies': []}
args = {}
for line in file_in:
    if '#movielist' in line:
        match = re.match(r'name: *?(?P<name>[a-zA-Z0-9 ]*)', next(file_in))
        if match and match.group('name'):
            data['title'] = match.group('name')
        continue

    match = re.match(
        r'^(?P<key>[a-zA-Z0-9 ]*):( *)?(?P<value>[a-zA-Z0-9 \.\:]*)', line)
    if match and match.group('key') and match.group('value'):
        args[match.group('key')] = match.group('value')

    if '#movie' in line:
        try:
            movie = Movie(args)
            data['movies'].append(movie)
        except TypeError:
            pass
        args = {}

try:
    movie = Movie(args)
    data['movies'].append(movie)
except TypeError:
    pass

data['movies'] = sorted(data['movies'], key=lambda m: (m.name, m.date))

# Transform to dictionary
movies = {}
for movie in data['movies']:
    if movie.name in movies:
        movies[movie.name]['watched'].append(movie.date)
    else:
        movies[movie.name] = {
            'watched': [movie.date],
            'file': movie.file
        }

if file_out:
    def to_li(name, data):
        return '<li>%s (Angesehen: %s, %s)</li>' % (
            name, len(data['watched']), data['file'])

    print('<!DOCTYPE html>', file=file_out)
    print('<meta charset="utf-8">', file=file_out)
    print('<h1>%s</h1>' % data['title'], file=file_out)
    print('<ul>', file=file_out)
    for movie in movies:
        print(to_li(movie, movies[movie]), file=file_out)
    print('</ul>', file=file_out)

if file_in:
    file_in.close()
if file_out:
    file_out.close()
